% EVMASOMAIN	Main program for
%
% 	evolutionary self-organization of potential field map
% 	by group of autonomous robots.
                        
%       Copyright (c) 1998-2000 Jiming Liu and Jianbing Wu

clear all;
close all;

global RobotIndex FreeIndex ObstacleIndex TRUE FALSE; 
global RealEnviron MapGridWidth EnvironSize MapSize ColorMap;
global RobotNum SensorType SensorNum RobotConfig MaxMoveStep;
global VisitTimes PotentFunc VisitPlusEnviron MaxRunStep;
global MarkColorIndex MaxAttitude TempPotentWeight;
global AccumulateNum TempPotentField PotentTemp BreakRatio;
global TotalPotent AllRobotConfig PrecisePotent;
global EnvThresh DirectionWeight IncrementWeight TotalWeight;
global FitnessFun PopulationSize CrossProb MutationProb SingleCodeWidth;
global MoveablePos MoveableNum VisitedWeight GenerationSize;

% --- Create GRAPHICS Windows ---
%defaultPos=get(0,'DefaultFigurePosition');
%defaultPos(1:2)=[440 500];
%defaultPos(3:4)=[350 350];
%set(0,'DefaultFigurePosition',defaultPos);
%FigHandleMap=figure(...
%            'Resize','off',...
%            'NumberTitle','off',...
%            'Name','Potential Field MAP');

%defaultPos(1:2)=[800 500];   
%set(0,'DefaultFigurePosition',defaultPos);
%FigHandleImg=figure(...
%            'Resize','off',...
%            'NumberTitle','off',...
%            'Name','Robotic Environment');
%clear defaultPos;

% ---- Initialization ----
globini;
robotini;
getenvir;
getrobot;
gaini;

BeginTime=clock;
%showmap(FigHandleMap,FigHandleImg);

% --- Running ---
AllRobotConfig=RobotConfig;
LastMoveDirect=ones(1,RobotNum)*(-1);
for RunTimes=1:MaxRunStep
  RunTimes            %%%%%%%%%%%%%%%

  SensingResult=sensing;
  RobotRunList=whorun;
  if isempty(RobotRunList)
    break;
  end

  [Careless,RunningNum]=size(RobotRunList);
  TotalRunningRobot(RunTimes,1:RunningNum)=RobotRunList;

  for loops=1:RunningNum
    SelectedRobot=RobotRunList(loops);
    ThisSensingResult=SensingResult(:,SelectedRobot);
    callsom(ThisSensingResult,SelectedRobot);
  end

  PotentTemp=getpotent;
  for mm=1:MapSize(1)
    for nn=1:MapSize(2)
      if ~PotentTemp(mm,nn)
        PotentTemp(mm,nn)=MaxAttitude;
      end
      if PotentTemp(mm,nn)>MaxAttitude
        PotentTemp(mm,nn)=MaxAttitude;
      end
    end
  end

  TotalPotent=[TotalPotent;PotentTemp];

%  showmap(FigHandleMap,FigHandleImg);

  [MoveableNum,MoveablePos]=moveablepos(RobotRunList);

  CheckStimulus=checkstimu(RobotRunList,SensingResult);

  SequRunList=[];  
  NewConfig=[];
  ThisMoveDirect=[];
  CoverList=RobotRunList;
  CovStimulus=CheckStimulus;  
  isNoCooper=0;
  while ~isNoCooper
    [Useless,UnRunNum]=size(CoverList);
    CoopList=[];
    CoopStimulus=[];
    CoopLastDirect=[];

    for loop1=1:UnRunNum-1
      for loop2=loop1+1:UnRunNum
        RNum1=CoverList(loop1);
        RNum2=CoverList(loop2); 
        ThisDistance=caldist(RobotConfig(RNum1,:),RobotConfig(RNum2,:));
        if ThisDistance<=EnvThresh
          if isempty(CoopList)
            CoopList=[RNum1 RNum2];
            CoopStimulus=[CovStimulus(:,loop1) CovStimulus(:,loop2)];
            CoopLastDirect=[LastMoveDirect(RNum1) LastMoveDirect(RNum2)];
          else
            if any(CoopList==RNum1) & all(CoopList~=RNum2)
              CoopList=[CoopList RNum2];
              CoopStimulus=[CoopStimulus CovStimulus(:,loop2)];
              CoopLastDirect=[CoopLastDirect LastMoveDirect(RNum2)];
            elseif any(CoopList==RNum2) & all(CoopList~=RNum1)                            
              CoopList=[CoopList RNum1];
              CoopStimulus=[CoopStimulus CovStimulus(:,loop1)];
              CoopLastDirect=[CoopLastDirect LastMoveDirect(RNum1)];
            end
          end
        end
      end
    end
    if ~isempty(CoopList)
      SequRunList=[SequRunList CoopList];
      for loops=UnRunNum:-1:1
        if any(CoopList==CoverList(loops))
          CoverList(loops)=[];
          CovStimulus(:,loops)=[];
        end
      end
    
      [ThisNewConfig,MoveDirect]=genmain(CoopList,CoopStimulus,NewConfig,...
             CoopLastDirect,1);
      NewConfig=[NewConfig;ThisNewConfig];
      ThisMoveDirect=[ThisMoveDirect MoveDirect];
    else
      isNoCooper=1;
      if ~isempty(CoverList)
        [Useless,CovNum]=size(CoverList);
        CovLastDirect=[];
        for loops=1:CovNum
          ThisRobot=CoverList(loops);
          CovLastDirect=[CovLastDirect LastMoveDirect(ThisRobot)];
        end
        SequRunList=[SequRunList CoverList];
        for loopcov=1:CovNum
          [ThisNewConfig,MoveDirect]=genmain(CoverList(loopcov),...
             CovStimulus(:,loopcov),NewConfig,CovLastDirect(loopcov),2);
          NewConfig=[NewConfig;ThisNewConfig];
          ThisMoveDirect=[ThisMoveDirect MoveDirect];
        end
      end
    end
  end

  LastMoveDirect=ones(1,RobotNum)*(-1);

  for loop1=1:RobotNum
    isInRunning=0;
    for loop2=1:RunningNum
      if SequRunList(loop2)==loop1
        isInRunning=1;
        ThisConfig=NewConfig(loop2,:);
        LastMoveDirect(loop1)=ThisMoveDirect(loop2);
        break;
      end
    end

    if isInRunning & all(ThisConfig~=[-1 -1])
      RobotConfig(loop1,:)=ThisConfig;
      AllRobotConfig=[AllRobotConfig;ThisConfig];
    else
      AllRobotConfig=[AllRobotConfig;-1 -1];
    end
  end

end

EndTime=clock;

[SaveDirectory,SaveItem]=saveini;
eval(['save ' SaveDirectory ' ' SaveItem ';']);
